/*
 * Column.cpp
 *
 *  Created on: Sep 28, 2008
 *      Author: Santi Ontan
 */

#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif


#ifdef _WIN32
#include "windows.h"
#endif

#include "stdlib.h"
#include "string.h"
#include "math.h"

#include "debug.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "auxiliar.h"
#include "2DCMC.h"
#include "Symbol.h"
#include "GLTile.h"
#include "keyboardstate.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "ColumnComplete.h"

#include "KObjectCreator.h"


ColumnComplete::ColumnComplete(int x,int y,int z) : K3DObject(x,y,z,0) {
    m_class = columncomplete_symbol;

	int i;
	float a;
	float tmpx,tmpy;
	float dx = 40, dy = 40, dz = 120;
	int divisions = 16;

	m_npoints = divisions*2+2;
	m_nfaces = divisions*4;
	m_points = new float[m_npoints*3];
	m_faces = new int[m_nfaces*3];;

	for(i=0;i<divisions;i++) {
		a = ((M_PI*2)/divisions)*i;
		tmpx = (float)cos(a)*dx/2;
		tmpy = (float)sin(a)*dy/2;

		m_points[(i*2)*3]=tmpx;
		m_points[(i*2)*3+1]=tmpy;
		m_points[(i*2)*3+2]=-dz/2;

		m_points[(i*2+1)*3]=tmpx;
		m_points[(i*2+1)*3+1]=tmpy;
		m_points[(i*2+1)*3+2]=dz/2-15;
	} //for

	m_points[(i*2)*3]=0;
	m_points[(i*2)*3+1]=0;
	m_points[(i*2)*3+2]=-dz/2;

	m_points[(i*2+1)*3]=0;
	m_points[(i*2+1)*3+1]=0;
	m_points[(i*2+1)*3+2]=dz/2-15;

	for(i=0;i<divisions;i++) {
		m_faces[(i*2)*3] = (i*2)%(divisions*2);
		m_faces[(i*2)*3+1] = ((i+1)*2)%(divisions*2);
		m_faces[(i*2)*3+2] = (i*2+1)%(divisions*2);

		m_faces[(i*2+1)*3] = (i*2+1)%(divisions*2);
		m_faces[(i*2+1)*3+1] = ((i+1)*2)%(divisions*2);
		m_faces[(i*2+1)*3+2] = ((i+1)*2+1)%(divisions*2);
	} // for

	// top part
	for(i=0;i<divisions;i++) {
		m_faces[(divisions*2+i)*3] = (divisions*2);
		m_faces[(divisions*2+i)*3+1] = ((i+1)*2)%(divisions*2);
		m_faces[(divisions*2+i)*3+2] =   (i*2)%(divisions*2);

		m_faces[(divisions*3+i)*3] = (divisions*2)+1;
		m_faces[(divisions*3+i)*3+1] = ((i+1)*2+1)%(divisions*2);
		m_faces[(divisions*3+i)*3+2] =   (i*2+1)%(divisions*2);
	} // for

	computeNormals();
	computeEdges(3.141592f/4.0f);

	m_r = 0.4f;
	m_g = 0.4f;
	m_b = 0.4f;

	K3DObject *o = new K3DObject(0,0,dz/2-3,0);
	o->createCube(dx,dy,3,0.4f,0.4f,0.4f,1.0f);
	m_parts.Add(o);

	o = new K3DObject(0,0,dz/2-7,0);
	o->createCube(dx-5,dy,3,0.4f,0.4f,0.4f,1.0f);
	m_parts.Add(o);

	o = new K3DObject(0,0,dz/2-12,0);
	o->createCube(dx-10,dy,3,0.4f,0.4f,0.4f,1.0f);
	m_parts.Add(o);

	o = new K3DObject(0,0,-dz/2-8,0);
	o->createCube(dx-16,dy,8,0.4f,0.4f,0.4f,1.0f);
	m_parts.Add(o);

	o = new K3DObject(0,0,-dz/2-18,0);
	o->createCube(dx-20,dy,2,0.4f,0.4f,0.4f,1.0f);
	m_parts.Add(o);
	
	// Create a fake "last_tile_used" for collision purposes:
	SDL_Surface *sfc = SDL_CreateRGBSurface(SDL_SWSURFACE, dx*2, dy*2, 32, RMASK, GMASK, BMASK, AMASK);
	SDL_FillRect(sfc,0,SDL_MapRGBA(sfc->format,255,255,255,255));
	collisionTile = new GLTile(sfc);
	collisionTile->set_hotspot(40, 40);
	collisionTile->set_cmc(0, 0, dx*2, dy*2);
	m_last_tile_used = collisionTile;	
}


ColumnComplete::~ColumnComplete() {
}


bool ColumnComplete::is_a(Symbol *c)
{
    if (c->cmp(columncomplete_symbol))
        return true;

    return K3DObject::is_a(c);
} /* ColumnComplete::is_a */


bool ColumnComplete::is_a(char *c)
{
    bool retval;
    Symbol *s = new Symbol(c);

    retval = is_a(s);

    delete s;

    return retval;
} /* ColumnComplete::is_a */
